#ifndef NETFS_H
#define NETFS_H

#include <inttypes.h>
#include <sys/types.h>
#include <tcp.h>

#define netfs_handle_t unsigned int

struct netfs_error_type {
	char *name;
	struct netfs_error_type *parent;
};

struct netfs_error {
	struct netfs_error_type *type;
	char *message;
};

struct netfs_connection {
	TCP_CONNECTION conn;
	size_t buffer_begin;
	size_t buffer_end;
	size_t buffer_size;
	uint8_t *buffer;
};

/* netfs_error is instantiated in every process */
#ifndef LIB
struct netfs_error *netfs_error = 0;
#endif

uint8_t *int2bignum(unsigned int in, uint8_t *out, size_t *size);
unsigned int bignum2int(const uint8_t *in, size_t *size);

struct netfs_connection *netfs_connect(const char *host, short port);
void netfs_disconnect(struct netfs_connection *conn);
void netfs_quit(struct netfs_connection *conn);
int netfs_negotiateProtocol(struct netfs_connection *conn);
int netfs_send(const struct netfs_connection *conn, const void *data,
		size_t size);
void *netfs_recv(struct netfs_connection *conn, void *data, size_t *size);
int netfs_sendInt(const struct netfs_connection *conn, unsigned int val);
unsigned int netfs_recvInt(struct netfs_connection *conn, size_t *size);

netfs_handle_t netfs_open(struct netfs_connection *conn,
	const char *name, int mode);
#define NETFS_O_RDONLY 1
#define NETFS_O_WRONLY 2
#define NETFS_O_RDWR 3
int netfs_close(struct netfs_connection *conn, netfs_handle_t handle);
void *netfs_read(struct netfs_connection *conn, netfs_handle_t handle,
	void *buffer, size_t *size);

#endif /* NETFS_H */
