#include "netfs.h"
#include <linux/spinlock.h>

/* From root.c */
extern int netfs_root_readdir(struct file *file, void *dirent, filldir_t fill);
extern struct file_operations netfs_root_fops;
extern struct inode_operations netfs_root_iops;

static spinlock_t ino_lock = SPIN_LOCK_UNLOCKED;
static ino_t next_ino = ROOT_INO + 1;

void netfs_read_inode(struct inode *inode) {
	ino_t ino = inode->i_ino;

	/* For a directory */
	inode->i_op = &simple_dir_inode_operations;
	inode->i_fop = &simple_dir_operations;
	inode->i_mode = S_IFDIR | S_IRUGO | S_IXUGO;
	inode->i_nlink = 2;
	inode->i_mtime = inode->i_atime = inode->i_ctime = CURRENT_TIME;
	inode->i_blocks = 0;
	inode->i_blksize = inode->i_sb->s_blocksize;

	/* For the root directory */
	if(ino == ROOT_INO) {
		inode->i_mode |= S_IWUSR;
		inode->i_fop = &netfs_root_fops;
		inode->i_op = &netfs_root_iops;
	}
}

/** Get next available inode number */
ino_t netfs_new_ino(void) {
	ino_t ino;

	/* Get next available inode number */
	spin_lock(&ino_lock);
	ino = next_ino++;
	spin_unlock(&ino_lock);

	return ino;
}
