#include "netfs.h"
#include <linux/smp_lock.h>

/* From inode.c */
extern ino_t netfs_new_ino(void);

static int netfs_root_readdir(struct file *file, void *dirent, filldir_t fill) {
	
	/* Return directory entries until the sky falls down */
	while(1) {
		/* The first two entries are special */
		if(file->f_pos == 0) {
			if(fill(dirent, ".", 1, file->f_pos, ROOT_INO,
					DT_DIR) < 0) {
				break;
			}
			file->f_pos++;
		}
		if(file->f_pos == 1) {
			if(fill(dirent, "..", 2, file->f_pos, ROOT_INO,
					DT_DIR) < 0) {
				break;
			}
			file->f_pos++;
		}
		/* No files in here */
		break;
	}
	
	return 0;
}

struct file_operations netfs_root_fops = {
	.read		= generic_read_dir,
	.readdir	= netfs_root_readdir
};

/** Create a directory */
static int netfs_mkdir(struct inode *parent, struct dentry *dentry, int mode) {
	parent->i_nlink++;
	d_add(dentry, iget(parent->i_sb, netfs_new_ino()));
	return 0;
}

/** This is called when the VFS looks up a file in the root directory.
 * We simply create any file that is looked up. */
struct dentry *netfs_root_lookup(struct inode *dir, struct dentry *dentry,
		struct nameidata *nd) {
	
	lock_kernel();

	/* Check name length */
	if(dentry->d_name.len > NAME_MAX) {
		unlock_kernel();
		return ERR_PTR(-ENAMETOOLONG);
	}

	/* Create directory */
	netfs_mkdir(dir, dentry, 0755);

	unlock_kernel();
	return NULL;
}

struct inode_operations netfs_root_iops = {
	.lookup		= netfs_root_lookup
};
