'''Helper functions for writing netfs clients'''

import socket
from netfs import *

O_RDONLY = 1
O_WRONLY = 2
O_RDWR = 3

def negotiateProtocol(conn):
	'''Negotiate about the protocol version to be used'''
	conn.send('\x01\x00\x01\x01')
	msg = recvInt(conn)
	if msg != 1:
		conn.close()
		raise ProtocolError('Invalid reply from server')

	msg = recvInt(conn)
	if msg != 1:
		conn.close()
		raise ProtocolError('Protocol negotiation failed')

def connect(host, port = 6999):
	'''Connect to netfs server'''
	conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	conn.connect((host, port))

	msg = recvInt(conn)
	if msg != 1:
		conn.close()
		raise ProtocolError('Invalid greeting from server')

	msg = recvInt(conn)
	if msg != 1:
		negotiateProtocol(conn)

	return conn

def quit(conn):
	'''Tell the server we're leavingi, then close the connection'''
	conn.send('\x02')
	conn.close()

def open(conn, name, mode = O_RDONLY):
	'''Open the named file'''
	conn.send('\x10')
	sendInt(conn, len(name))
	conn.send(name)
	sendInt(conn, mode)
	msg = recvInt(conn)
	if msg == 0:
		msg = recvInt(conn)
		if msg == 1:
			raise FileError('File not found')
		elif msg == 2:
			raise FileError('Access denied')
		else:
			raise FileError('Error opening file')

	return msg

def close(conn, handle):
	'''Close an open file'''
	conn.send('\x11')
	sendInt(conn, handle)
	msg = recvInt(conn)
	if msg == 0:
		return
	elif msg == 1:
		raise FileError('Invalid file handle')
	else:
		raise FileError()	

def read(conn, handle, n):
	'''Read from file'''
	conn.send('\x12')
	sendInt(conn, handle)
	sendInt(conn, n)
	n = recvInt(conn)
	if n == 0:
		msg = recvInt(conn)
		if msg == 0:
			return ''
		elif msg == 1:
			raise FileError('Invalid file handle')
		else:
			raise FileError
	
	return conn.recv(n)
