#! /usr/bin/env python

HOST = ''
PORT = 6999

import sys, socket, thread, netfs

clients = []

class Client:
	def __init__(self, conn, addr):
		self.conn = conn
		self.addr = addr
		self.files = []
		clients.append(self)
		
	def send(self, data):
		return self.conn.send(data)

	def recv(self, n):
		return self.conn.recv(n)

	def sendInt(self, val):
		return netfs.sendInt(self.conn, val)

	def recvInt(self):
		return netfs.recvInt(self.conn)

	def shutdown(self):
		self.conn.close()
		clients.remove(self)

def cmd_open(c):
	n = c.recvInt()
	name = c.recv(n)
	n = c.recvInt()
	if n == 1: mode = 'r'
	elif n == 2: mode = 'w'
	elif n == 3: mode = 'rw'
	else:
		c.send('\x00\x02')
		return False

	# For now, disallow writing
	if mode != 'r':
		c.send('\x00\x02')
		return False

	# Find vacancy in file list
	n = 0
	for i in c.files:
		n = n + 1
		if i == False:
			break

	if n == len(c.files):
		c.files.append(False)
		n = n + 1

	# Open file
	try:
		c.files[n - 1] = open(name, mode)
	except IOError:
		c.send('\x00\x02')
		return False

	c.send(chr(n))
	return True

def cmd_close(c):
	# Get handle
	n = c.recvInt()
	if n > len(c.files):
		c.send('\x01')
		return False

	# Close file
	try:
		c.files[n - 1].close()
	except:
		c.send('\x02')
		return False

	c.files[n - 1] = False
	c.send('\x00')
	return True

def cmd_read(c):
	# Get handle and count
	handle = c.recvInt()
	n = c.recvInt()

	# Verify handle
	if handle > len(c.files):
		c.send('\x00\x01')
		return False

	# Read data
	try:
		data = c.files[handle - 1].read(n)
	except:
		c.send('\x00\x02')
		return False

	# If no data, we have reached EOF
	if len(data) == 0:
		c.send('\x00\x00')
		return True

	# Send data
	c.sendInt(len(data))
	c.send(data)
	return True

def cmd_write(c):
	# Get handle and data
	handle = c.recvInt()
	n = c.recvInt()
	data = c.recv(n)

	# Verify handle
	if handle > len(c.files):
		c.send('\x00\x01')
		return False

	# Write data
	try:
		c.files[handle - 1].write(data)
	except:
		c.send('\x00\x02')
		return False
	return True

def cmd_pos(c):
	# Read handle and position
	handle = c.recvInt()
	pos = c.recvInt()

	# Verify handle
	if handle > len(c.files):
		c.send('\x01')
		return False

	# Seek
	try:
		c.files[handle - 1].seek(pos)
	except:
		c.send('\x02')
		return False

	c.send('\x00')
	return True

def cmd_tell(c):
	# Read handle
	handle = c.recvInt()

	# Verify handle
	if handle > len(c.files):
		c.send('\x01')
		return False

	# Get position
	try:
		pos = c.files[handle - 1].tell()
	except:
		c.send('\x02')
		return False

	c.send('\x00')
	c.sendInt(pos)
	return True

def cmd_seek(c):
	# Read handle, position, and whence
	handle = c.recvInt()
	pos = c.recvInt()
	whence = c.recvInt()

	# Verify handle
	if handle > len(c.files):
		c.send('\x01')
		return False

	# Seek
	try:
		c.files[handle - 1].seek(pos, whence)
	except:
		c.send('\x02')
		return False

	c.send('\x00')
	return True

def handle_connection(conn, addr):
	# Create client entity
	c = Client(conn, addr)

	# Say hello
	try:
		c.send('\x01\x01')

		while True:
			op = c.recvInt()
			if op == 1:
				# We don't want any protocol negotiation
				break
			elif op == 2:
				break
			elif op == 0x10:
				cmd_open(c)
			elif op == 0x11:
				cmd_close(c)
			elif op == 0x12:
				cmd_read(c)
			elif op == 0x13:
				cmd_write(c)
			elif op == 0x14:
				cmd_pos(c)
			elif op == 0x15:
				cmd_tell(c)
			elif op == 0x16:
				cmd_seek(c)
			else:
				# Protocol error
				break
	except:
		pass

	c.shutdown()

def shutdown():
	sock.close()
	for c in clients:
		c.shutdown()

sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.bind((HOST, PORT))
sock.listen(3)

# Main loop
while True:
	try:
		thread.start_new_thread(handle_connection, sock.accept())
	except KeyboardInterrupt:
		shutdown()
		break
